Attribute VB_Name = "modVBECode"
Option Compare Database
Option Explicit

Public strModule As String

'###############################################
'64-bit declarations checked 05/03/2019
#If VBA7 Then
    Private Declare PtrSafe Function FindWindow Lib "user32" Alias "FindWindowA" _
            (ByVal ClassName As String, ByVal WindowName As String) As LongPtr
        
    Private Declare PtrSafe Function LockWindowUpdate Lib "user32" _
            (ByVal hWndLock As LongPtr) As Long

#Else '32-bit Office
    Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" _
            (ByVal ClassName As String, ByVal WindowName As String) As Long
        
    Private Declare Function LockWindowUpdate Lib "user32" _
            (ByVal hWndLock As Long) As Long

#End If
'###############################################

'General Infomation About A Procedure
'The code below returns the following information about a procedure in a module, loaded into the ProcInfo Type.
'The function ProcedureInfo takes as input then name of the procedure, a VBIDE.vbext_ProcKind procedure type, and a reference to the CodeModule object containing the procedure.

    Public Enum ProcScope
        ScopePrivate = 1
        ScopePublic = 2
        ScopeFriend = 3
        ScopeDefault = 4
    End Enum
    
    Public Enum LineSplits
        LineSplitRemove = 0
        LineSplitKeep = 1
        LineSplitConvert = 2
    End Enum
    
    Public Type ProcInfo
        ProcName As String
        ProcKind As VBIDE.vbext_ProcKind
        ProcStartLine As Long
        ProcBodyLine As Long
        ProcCountLines As Long
        ProcScope As ProcScope
        ProcDeclaration As String
    End Type

'###################################################################
'www.cpearson.com      chip@cpearson.com Office Integration ProjectsNET ProgrammingXML Development

'You can write code in VBA that reads or modifies other VBA projects, modules, or procedures.
'This is called extensibility because extends the editor -- you can use VBA code to create new VBA code.
'You can use these features to write custom procedures that create, change, or delete VBA modules and code procedures.
'First, set an reference to the VBA Extensibility library.

'Objects In The VBA Extensibility Model

'The following is a list of the more common objects that are used in the VBA Extensibilty object model. This is not a comprehensive list, but will be sufficient for the tasks at hand.

'VBIDE
'The VBIDE is the object library that defines all the objects and values that make up VBProject and the Visual Basic Editor.
'You must reference this library to use the VBA Extensibility objects.
'To add this reference, open the VBA editor, open your VBProject in the editor, and go to the Tools menu.
'There, choose References . In the References dialog, scroll down to Microsoft Visual Basic for Applications Extensibility 5.3 and check that item in the list. You can add the reference programmatically with code like:

    'ThisWorkbook.VBProject.References.AddFromGuid
    '    GUID:="{0002E157-0000-0000-C000-000000000046}", _
    '    Major:=5, Minor:=3

'VBE
'The VBE refers to the Visual Basic Editor, which includes all the windows and projects that make up the editor.

'VBProject
'A VBProject contains all the code modules and components of a single database.
'One database has exactly one VBProject.
'The VBProject is made up of 1 or more VBComponent objects.

'VBComponent
'A VBComponent is one object within the VBProject. A VBComponent is a regular code module, a UserForm, a class module, a report etc).
'A VBComponent is of one of the following types, identified by the Type property. The following constants are used to identify the Type. The numeric value of each constant is shown in parentheses.
'vbext_ct_ClassModule (2): A class module to create your own objects. See Class Modules for details about classes and objects.
'vbext_ct_Document (100): One of the Sheet modules or the ThisWorkbook module.
'vbext_ct_MSForm (3): A UserForm. The visual component of a UserForm in the VBA Editor is called a Designer.
'vbext_ct_StdModule (1): A regular code module. Most of the procedures on this page will work with these types of components.

'CodeModule
'A CodeModule is the VBA source code of a VBComponent. You use the CodeModule object to access the code associated with a VBComponent. A VBComponent has exactly one CodeModule which contains all the code for that component.

'CodePane
'A CodePane is an open editing window of a CodeModule. When you are typing code, you are entering code into the CodePane.

'Referencing VBIDE Objects
'The code below illustrate various ways to reference Extensibility objects.
'Dim VBAEditor As VBIDE.VBE
'Dim VBProj As VBIDE.VBProject
'Dim VBComp As VBIDE.VBComponent
'Dim CodeMod As VBIDE.CodeModule

'Set VBAEditor = Application.VBE
'Set VBProj = VBAEditor.ActiveVBProject
'Set VBComp = VBProj.VBComponents("Module1")
'Set CodeMod = VBComp.CodeModule

'In the code and descriptions on this page, the term Procedure means a Sub, Function,Property Get, Property Let, or Property Set procedure.
'The Extensibility library defines four procedures types, identified by the following constants.
'The numeric value of each constant is shown within parentheses.

'vbext_pk_Get (3). A Property Get procedure.
'vbext_pk_Let (1). A Property Let procedure.
'vbext_pk_Set (2). A Property Set procedure.
'vbext_pk_Proc (0). A Sub or Function procedure.

'The rest of this module describes various procedures that modify the various objects of a VBProject.

'Ensuring The Editor In Synchronized
'The VBA editor is said to be "in sync" if the ActiveVBProject is the same as the VBProject that contains the ActiveCodePane.
'If you have two or more projects open within the VBA editor, it is possible to have an active code pane open from Project1 and have a component of Project2 selected in the Project Explorer window. In this case, the Application.VBE.ActiveVBProject is the project that is selected in the Project window, while Application.VBE.ActiveCodePane is a different project, specifically the project referenced by Application.VBE.ActiveCodePane.CodeModule.Parent.Collection.Parent.

'#################################
'You can test whether the editor in in sync with code like the following.

Function IsEditorInSync() As Boolean
'=======================================================================
' IsEditorInSync
' This tests if the VBProject selected in the Project window, and
' therefore the ActiveVBProject is the same as the VBProject associated
' with the ActiveCodePane. If these two VBProjects are the same,
' the editor is in sync and the result is True. If these are not the
' same project, the editor is out of sync and the result is True.
'=======================================================================
    With Application.VBE
    IsEditorInSync = .ActiveVBProject Is _
        .ActiveCodePane.CodeModule.Parent.Collection.Parent
    End With
End Function

'#################################
'You can force synchronization with code like the following.
'This will set the ActiveVBProject to the project associated with the ActiveCodePane.

Sub SyncVBAEditor()
'=======================================================================
' SyncVBAEditor
' This syncs the editor with respect to the ActiveVBProject and the
' VBProject containing the ActiveCodePane. This makes the project
' that contains the ActiveCodePane the ActiveVBProject.
'=======================================================================
With Application.VBE
If Not .ActiveCodePane Is Nothing Then
    Set .ActiveVBProject = .ActiveCodePane.CodeModule.Parent.Collection.Parent
End If
End With
End Sub
'######################

'#################################
'Adding A Module To A Project

'This code will add new code module named NewModule to the current VBProject.
'The type of VBComponent is specified by the value of the parameter passed to the Add method.

    Sub AddModuleToProject()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents.Add(vbext_ct_StdModule)
        VBComp.Name = "NewModule"
    End Sub
    
'#################################
'Adding A Procedure To A Module
'Creating a procedure via VBA code is really quite simple. Build up a text string of the code, using vbCrLf to create new lines, and then insert that text with the InsertLines method, passing to it the line number and the text string. The following code will add a simple "Hello World" procedure named SayHello to the end of the module named Module1.

    Sub AddProcedureToModule()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim LineNum As Long
        Const DQUOTE = """" ' one " character

        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents("Module1")
        Set CodeMod = VBComp.CodeModule
        
        With CodeMod
            LineNum = .CountOfLines + 1
            .InsertLines LineNum, "Public Sub SayHello()"
            LineNum = LineNum + 1
            .InsertLines LineNum, "    MsgBox " & DQUOTE & "Hello World" & DQUOTE
            LineNum = LineNum + 1
            .InsertLines LineNum, "End Sub"
        End With
    End Sub
    
'#################################
'You can also build up a String variable with the content of the procedure and insert that string with one call to InsertLines.
'For example,

'    Dim CodePan As VBIDE.CodeModule
'    Dim S As String
'    Set CodePan = CurrentDB.VBProject.VBComponents("Module2").CodeModule
'    S = "Sub ABC()" & vbNewLine & _
        "    MsgBox ""Hello World"",vbOkOnly" & vbNewLine & _
'    "End Sub" & vbNewLine
    
'    With CodePan
'        .InsertLines .CountOfLines + 1, S
'    End With

'#################################
'Copy A Module From One Project To Another
'There is no direct way to copy a module from one project to another.
'To accomplish this task, you must export the module from the Source VBProject and then import that file into the Destination VBProject. The code below will do this. The function declaration is:

'Function CopyModule(ModuleName As String, _
    FromVBProject As VBIDE.VBProject, _
    ToVBProject As VBIDE.VBProject, _
    OverwriteExisting As Boolean) As Boolean

'ModuleName is the name of the module you want to copy from one project to another.

'FromVBProject is the VBProject that contains the module to be copied. This is the source VBProject.

'ToVBProject is the VBProject in to which the module is to be copied. This is the destination VBProject.

'OverwriteExisting indicates what to do if ModuleName already exists in the ToVBProject. If this is True the existing VBComponent will be removed from the ToVBProject. If this is False and the VBComponent already exists, the function does nothing and returns False.

'The function returns True if successful or False is an error occurs.
    'The function will return False if any of the following are true:
    'FromVBProject is nothing.
    'ToVBProject is nothing.
    'ModuleName is blank.
    'FromVBProject is locked.
    'ToVBProject is locked.
    'ModuleName does not exist in FromVBProject.
    'ModuleName exists in ToVBProject and OverwriteExisting is False.

'The complete code is shown below:

Function CopyModule(ModuleName As String, _
    FromVBProject As VBIDE.VBProject, _
    ToVBProject As VBIDE.VBProject, _
    OverwriteExisting As Boolean) As Boolean
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' CopyModule
    ' This function copies a module from one VBProject to
    ' another. It returns True if successful or  False
    ' if an error occurs.
    '
    ' Parameters:
    ' --------------------------------
    ' FromVBProject         The VBProject that contains the module
    '                       to be copied.
    '
    ' ToVBProject           The VBProject into which the module is
    '                       to be copied.
    '
    ' ModuleName            The name of the module to copy.
    '
    ' OverwriteExisting     If True, the VBComponent named ModuleName
    '                       in ToVBProject will be removed before
    '                       importing the module. If False and
    '                       a VBComponent named ModuleName exists
    '                       in ToVBProject, the code will return
    '                       False.
    '
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    
    Dim VBComp As VBIDE.VBComponent
    Dim FName As String
    Dim CompName As String
    Dim S As String
    Dim SlashPos As Long
    Dim ExtPos As Long
    Dim TempVBComp As VBIDE.VBComponent
    
    '''''''''''''''''''''''''''''''''''''''''''''
    ' Do some housekeeping validation.
    '''''''''''''''''''''''''''''''''''''''''''''
    If FromVBProject Is Nothing Then
        CopyModule = False
        Exit Function
    End If
    
    If Trim(ModuleName) = vbNullString Then
        CopyModule = False
        Exit Function
    End If
    
    If ToVBProject Is Nothing Then
        CopyModule = False
        Exit Function
    End If
    
    If FromVBProject.Protection = vbext_pp_locked Then
        CopyModule = False
        Exit Function
    End If
    
    If ToVBProject.Protection = vbext_pp_locked Then
        CopyModule = False
        Exit Function
    End If
    
    On Error Resume Next
    Set VBComp = FromVBProject.VBComponents(ModuleName)
    If Err.Number <> 0 Then
        CopyModule = False
        Exit Function
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' FName is the name of the temporary file to be
    ' used in the Export/Import code.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''
    FName = Environ("Temp") & "\" & ModuleName & ".bas"
    If OverwriteExisting = True Then
        ''''''''''''''''''''''''''''''''''''''
        ' If OverwriteExisting is True, Kill
        ' the existing temp file and remove
        ' the existing VBComponent from the
        ' ToVBProject.
        ''''''''''''''''''''''''''''''''''''''
        If Dir(FName, vbNormal + vbHidden + vbSystem) <> vbNullString Then
            Err.Clear
            Kill FName
            If Err.Number <> 0 Then
                CopyModule = False
                Exit Function
            End If
        End If
        With ToVBProject.VBComponents
            .Remove .Item(ModuleName)
        End With
    Else
        '''''''''''''''''''''''''''''''''''''''''
        ' OverwriteExisting is False. If there is
        ' already a VBComponent named ModuleName,
        ' exit with a return code of False.
        ''''''''''''''''''''''''''''''''''''''''''
        Err.Clear
        Set VBComp = ToVBProject.VBComponents(ModuleName)
        If Err.Number <> 0 Then
            If Err.Number = 9 Then
                ' module doesn't exist. ignore error.
            Else
                ' other error. get out with return value of False
                CopyModule = False
                Exit Function
            End If
        End If
    End If
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' Do the Export and Import operation using FName
    ' and then Kill FName.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''
    FromVBProject.VBComponents(ModuleName).Export FileName:=FName
    
    '''''''''''''''''''''''''''''''''''''
    ' Extract the module name from the
    ' export file name.
    '''''''''''''''''''''''''''''''''''''
    SlashPos = InStrRev(FName, "\")
    ExtPos = InStrRev(FName, ".")
    CompName = Mid(FName, SlashPos + 1, ExtPos - SlashPos - 1)
    
    ''''''''''''''''''''''''''''''''''''''''''''''
    ' Document modules (SheetX and ThisWorkbook)
    ' cannot be removed. So, if we are working with
    ' a document object, delete all code in that
    ' component and add the lines of FName
    ' back in to the module.
    ''''''''''''''''''''''''''''''''''''''''''''''
    Set VBComp = Nothing
    Set VBComp = ToVBProject.VBComponents(CompName)
    
    If VBComp Is Nothing Then
        ToVBProject.VBComponents.Import FileName:=FName
    Else
        If VBComp.type = vbext_ct_Document Then
            ' VBComp is destination module
            Set TempVBComp = ToVBProject.VBComponents.Import(FName)
            ' TempVBComp is source module
            With VBComp.CodeModule
                .DeleteLines 1, .CountOfLines
                S = TempVBComp.CodeModule.Lines(1, TempVBComp.CodeModule.CountOfLines)
                .InsertLines 1, S
            End With
            On Error GoTo 0
            ToVBProject.VBComponents.Remove TempVBComp
        End If
    End If
    Kill FName
    CopyModule = True
End Function

'#################################
'Creating An Event Procedure
'This code will create a Module_Open event procedure.
'When creating an event procedure, you should use the CreateEventProc method so that the correct procedure declaration and parameter list is used. CreateEventProc will create the declaration line and the end of procedure line. It returns the line number on which the event procedure begins.

    Sub CreateEventProcedure()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim LineNum As Long
        Const DQUOTE = """" ' one " character

        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents("Module1")
        Set CodeMod = VBComp.CodeModule
        
        With CodeMod
            LineNum = .CreateEventProc("Open", "Module1")
            LineNum = LineNum + 1
            .InsertLines LineNum, "    MsgBox " & DQUOTE & "Hello World" & DQUOTE
        End With
    End Sub

'#################################
'Creating A Procedure
'You can use code to create code in a module.
'The code below creates a simple "Hello World" Sub procedure.
'You can either create a new VBComponent to hold the procedure or you can use an existing module.
'Comment out the appropriate lines of code.

Sub CreateProcedure()
    Dim VBComp As VBIDE.VBComponent
    Dim CodeMod As VBIDE.CodeModule
    Dim S As String
    Dim LineNum As Long
    
    ' Use the next two lines to create a new module for the code
    'Set VBComp = VBProj.VBComponents.Add(vbext_ct_StdModule)
    'VBComp.Name = "NewModule"
    ' OR use the following line to use an existing module for the code
    'Set VBComp = VBProj.VBComponents("Module2")
    
    Set CodeMod = VBComp.CodeModule
    LineNum = CodeMod.CountOfLines + 1
    S = "Sub HelloWorld()" & vbCrLf & _
        "    MsgBox ""Hello, World""" & vbCrLf & _
        "End Sub"
    CodeMod.InsertLines LineNum, S
End Sub

'This code creates the procedure:

  '  Sub HelloWorld()
  '      MsgBox "Hello, World"
  '  End Sub

'#################################
'Deleting A Module From A Project
'This code will delete Module1 from the VBProject.
'In general, you cannot delete a module whose Type is vbext_ct_Document.

    Sub DeleteModule()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
    
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents("Module1")
        VBProj.VBComponents.Remove VBComp
    End Sub
Sub DeleteProcedureFromModule()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        
        Dim StartLine As Long
        Dim NumLines As Long
        Dim ProcName As String
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents("LibraryCode1")
        Set CodeMod = VBComp.CodeModule
    
        ProcName = "XEOYX"
        
        With CodeMod
            StartLine = .ProcStartLine(ProcName, vbext_pk_Proc)
            NumLines = .ProcCountLines(ProcName, vbext_pk_Proc)
            .DeleteLines StartLine:=StartLine, count:=NumLines
        End With
        
        MsgBox "DONE"
    End Sub
    
'######################
'Deleting All VBA Code In A Project
'This code will delete ALL VBA code in a VBProject.

    Sub DeleteAllVBACode()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        
        Set VBProj = VBAEditor.ActiveVBProject
        
        For Each VBComp In VBProj.VBComponents
            If VBComp.type = vbext_ct_Document Then
                Set CodeMod = VBComp.CodeModule
                With CodeMod
                    .DeleteLines 1, .CountOfLines
                End With
            Else
                VBProj.VBComponents.Remove VBComp
            End If
        Next VBComp
    End Sub
    
    
'######################
'Eliminating Screen Flicker During VBProject Code
'When you used the Extensibility code, the VBA Editor window will flicker. This can be reduced with the code:
    'Application.VBE.MainWindow.visible = False

'This will hide the VBE window, but you may still see it flicker.
'To prevent this, you must use the LockWindowUpdate Windows API function.
 '   Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" _
        (ByVal ClassName As String, ByVal WindowName As String) As Long
    
 '   Private Declare Function LockWindowUpdate Lib "user32" _
        (ByVal hWndLock As Long) As Long

Public Sub EliminateScreenFlicker()

'Updated 05/03/2019 for 64-bit compatability
#If VBA7 Then
    Dim VBEHwnd As LongPtr
#Else
    Dim VBEHwnd As Long
#End If
        
    On Error GoTo ErrH:
    
    Application.VBE.MainWindow.visible = False
    
    VBEHwnd = FindWindow("wndclass_desked_gsk", _
        Application.VBE.MainWindow.Caption)
    
    If VBEHwnd Then
        LockWindowUpdate VBEHwnd
    End If
    
    '''''''''''''''''''''''''
    ' your code here
    '''''''''''''''''''''''''
    
    Application.VBE.MainWindow.visible = False
    
ErrH:
    LockWindowUpdate 0&
End Sub
    
'######################
'Exporting A VBComponent Code Module To A Text File
'You can export an existing VBComponent CodeModule to a text file.
'This can be useful if you are archiving modules to create a library of useful module to be used in other projects.

    Public Function ExportVBComponent(VBComp As VBIDE.VBComponent, _
                FolderName As String, _
                Optional FileName As String, _
                Optional OverwriteExisting As Boolean = True) As Boolean
                
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' This function exports the code module of a VBComponent to a text
    ' file. If FileName is missing, the code will be exported to
    ' a file with the same name as the VBComponent followed by the
    ' appropriate extension.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Dim Extension As String
    Dim FName As String
    Extension = GetFileExtension(VBComp:=VBComp)
    If Trim(FileName) = vbNullString Then
        FName = VBComp.Name & Extension
    Else
        FName = FileName
        If InStr(1, FName, ".", vbBinaryCompare) = 0 Then
            FName = FName & Extension
        End If
    End If
    
    If StrComp(right(FolderName, 1), "\", vbBinaryCompare) = 0 Then
        FName = FolderName & FName
    Else
        FName = FolderName & "\" & FName
    End If
    
    If Dir(FName, vbNormal + vbHidden + vbSystem) <> vbNullString Then
        If OverwriteExisting = True Then
            Kill FName
        Else
            ExportVBComponent = False
            Exit Function
        End If
    End If
    
    VBComp.Export FileName:=FName
    ExportVBComponent = True
    
    End Function
    
    '#################################
    Public Function GetFileExtension(VBComp As VBIDE.VBComponent) As String
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' This returns the appropriate file extension based on the Type of
    ' the VBComponent.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Select Case VBComp.type
            Case vbext_ct_ClassModule
                GetFileExtension = ".cls"
            Case vbext_ct_Document
                GetFileExtension = ".cls"
            Case vbext_ct_MSForm
                GetFileExtension = ".frm"
            Case vbext_ct_StdModule
                GetFileExtension = ".bas"
            Case Else
                GetFileExtension = ".bas"
        End Select
        
    End Function
    
'######################
'Listing All Modules In A Project
'This code will list all the modules and their types in the project

    Sub ListStandardModules()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents("Module1")
        
        For Each VBComp In VBProj.VBComponents
            If Left(VBComp.Name, 4) <> "Form" And Left(VBComp.Name, 6) <> "Report" Then
                Debug.Print VBComp.Name
            End If
        Next VBComp
    End Sub
    
    Sub ListAllModules()
        'this includes code modules for forms & reports
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
      '  Set VBComp = VBProj.VBComponents("Module1")
        
        For Each VBComp In VBProj.VBComponents
            'If Left(VBComp.Name, 4) <> "Form" And Left(VBComp.Name, 6) <> "Report" Then
                Debug.Print VBComp.Name
            'End If
        Next VBComp
    End Sub

    Function ComponentTypeToString(ComponentType As VBIDE.vbext_ComponentType) As String
        Select Case ComponentType
            Case vbext_ct_ActiveXDesigner
                ComponentTypeToString = "ActiveX Designer"
            Case vbext_ct_ClassModule
                ComponentTypeToString = "Class Module"
            Case vbext_ct_Document
                ComponentTypeToString = "Document Module"
            Case vbext_ct_MSForm
                ComponentTypeToString = "UserForm"
            Case vbext_ct_StdModule
                ComponentTypeToString = "Code Module"
            Case Else
                ComponentTypeToString = "Unknown Type: " & CStr(ComponentType)
        End Select
    End Function
    
'######################
'Listing All Procedures In A Module
'This code will list all the procedures in the named module
'For example ListProcedures("modVBECode")

Sub ListProcedures(strModule As String)
    
On Error GoTo Err_Handler
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim LineNum As Long
        Dim NumLines As Long
        Dim ProcName As String
        Dim ProcKind As VBIDE.vbext_ProcKind
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents(strModule)
        Set CodeMod = VBComp.CodeModule
      
        With CodeMod
            LineNum = .CountOfDeclarationLines + 1
            Do Until LineNum >= .CountOfLines
                ProcName = .ProcOfLine(LineNum, ProcKind)
                LineNum = .ProcStartLine(ProcName, ProcKind) + _
                        .ProcCountLines(ProcName, ProcKind) + 1
              Debug.Print ProcName
            Loop
        End With
        
Exit_Handler:
        Exit Sub
        
Err_Handler:
    If Err = 9 Then
        MsgBox "Module does not exist", vbCritical, "No such module"
    Else
        MsgBox "Error " & Err.Number & " in ListProcedures : " & Err.Description
    End If
    GoTo Exit_Handler
    
End Sub

'This code will list all the procedures in standard modules

Sub ListAllStandardModuleProcedures()
    
On Error GoTo Err_Handler
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim LineNum As Long
        Dim NumLines As Long
        Dim ProcName As String
        Dim ProcKind As VBIDE.vbext_ProcKind
        
        Dim strCriteria As String, strSelection As String, strText As String
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        
        For Each VBComp In VBProj.VBComponents
            Set CodeMod = VBComp.CodeModule
            If Left(VBComp.Name, 4) <> "Form" And Left(VBComp.Name, 6) <> "Report" Then
               ' strSelection = VBComp.Name
                Debug.Print VBComp.Name
                With CodeMod
                    LineNum = .CountOfDeclarationLines + 1
                    Do Until LineNum >= .CountOfLines
                        ProcName = .ProcOfLine(LineNum, ProcKind)
                        LineNum = .ProcStartLine(ProcName, ProcKind) + _
                                .ProcCountLines(ProcName, ProcKind) + 1
                     
                        strCriteria = ProcName
                        
                        'check for duplicates due to Win64 code
                        If strText <> strCriteria Then
                            Debug.Print "         - " & ProcName
                            strText = strCriteria
                        End If
                    Loop
                End With
            End If
        Next VBComp
        
    'tidy up
    strSelection = ""
    strCriteria = ""
    strText = ""
                
Exit_Handler:
        Exit Sub
        
Err_Handler:
    MsgBox "Error " & Err.Number & " in ListAllStandardProcedures : " & Err.Description
    GoTo Exit_Handler
    
End Sub
    
'This code will list all the procedures in all modules
Sub ListAllModuleProcedures()
    
On Error GoTo Err_Handler
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim LineNum As Long
        Dim NumLines As Long
        Dim ProcName As String
        Dim ProcKind As VBIDE.vbext_ProcKind
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        
        For Each VBComp In VBProj.VBComponents
            Set CodeMod = VBComp.CodeModule
            Debug.Print VBComp.Name
            With CodeMod
                LineNum = .CountOfDeclarationLines + 1
                Do Until LineNum >= .CountOfLines
                    ProcName = .ProcOfLine(LineNum, ProcKind)
                    LineNum = .ProcStartLine(ProcName, ProcKind) + _
                            .ProcCountLines(ProcName, ProcKind) + 1
                  Debug.Print "         - " & ProcName
                Loop
            End With
        Next VBComp
                
Exit_Handler:
        Exit Sub
        
Err_Handler:
    MsgBox "Error " & Err.Number & " in ListAllStandardProcedures : " & Err.Description
    GoTo Exit_Handler
    
End Sub
    Function ProcKindString(ProcKind As VBIDE.vbext_ProcKind) As String
        Select Case ProcKind
            Case vbext_pk_Get
                ProcKindString = "Property Get"
            Case vbext_pk_Let
                ProcKindString = "Property Let"
            Case vbext_pk_Set
                ProcKindString = "Property Set"
            Case vbext_pk_Proc
                ProcKindString = "Sub Or Function"
            Case Else
                ProcKindString = "Unknown Type: " & CStr(ProcKind)
        End Select
    End Function

    Function ProcedureInfo(ProcName As String, ProcKind As VBIDE.vbext_ProcKind, _
        CodeMod As VBIDE.CodeModule) As ProcInfo
    
        Dim PInfo As ProcInfo
        Dim BodyLine As Long
        Dim Declaration As String
        Dim FirstLine As String
        
        BodyLine = CodeMod.ProcStartLine(ProcName, ProcKind)
        If BodyLine > 0 Then
            With CodeMod
                PInfo.ProcName = ProcName
                PInfo.ProcKind = ProcKind
                PInfo.ProcBodyLine = .ProcBodyLine(ProcName, ProcKind)
                PInfo.ProcCountLines = .ProcCountLines(ProcName, ProcKind)
                PInfo.ProcStartLine = .ProcStartLine(ProcName, ProcKind)
                
                FirstLine = .Lines(PInfo.ProcBodyLine, 1)
                If StrComp(Left(FirstLine, Len("Public")), "Public", vbBinaryCompare) = 0 Then
                    PInfo.ProcScope = ScopePublic
                ElseIf StrComp(Left(FirstLine, Len("Private")), "Private", vbBinaryCompare) = 0 Then
                    PInfo.ProcScope = ScopePrivate
                ElseIf StrComp(Left(FirstLine, Len("Friend")), "Friend", vbBinaryCompare) = 0 Then
                    PInfo.ProcScope = ScopeFriend
                Else
                    PInfo.ProcScope = ScopeDefault
                End If
                PInfo.ProcDeclaration = GetProcedureDeclaration(CodeMod, ProcName, ProcKind, LineSplitKeep)
            End With
        End If
        
        ProcedureInfo = PInfo
        
       ' Debug.Print PInfo
    
    End Function
    
    Public Function GetProcedureDeclaration(CodeMod As VBIDE.CodeModule, _
        ProcName As String, ProcKind As VBIDE.vbext_ProcKind, _
        Optional LineSplitBehavior As LineSplits = LineSplitConvert)
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' GetProcedureDeclaration
    ' This return the procedure declaration of ProcName in CodeMod. The LineSplitBehavior
    ' determines what to do with procedure declaration that span more than one line using
    ' the "_" line continuation character. If LineSplitBehavior is LineSplitRemove, the
    ' entire procedure declaration is converted to a single line of text. If
    ' LineSplitBehavior is LineSplitKeep the "_" characters are retained and the
    ' declaration is split with vbNewLine into multiple lines. If LineSplitBehavior is
    ' LineSplitConvert, the "_" characters are removed and replaced with vbNewLine.
    ' The function returns vbNullString if the procedure could not be found.
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Dim LineNum As Long
        Dim S As String
        Dim Declaration As String
        
        On Error Resume Next
        LineNum = CodeMod.ProcBodyLine(ProcName, ProcKind)
        If Err.Number <> 0 Then
            Exit Function
        End If
        S = CodeMod.Lines(LineNum, 1)
        Do While right(S, 1) = "_"
            Select Case True
                Case LineSplitBehavior = LineSplitConvert
                    S = Left(S, Len(S) - 1) & vbNewLine
                Case LineSplitBehavior = LineSplitKeep
                    S = S & vbNewLine
                Case LineSplitBehavior = LineSplitRemove
                    S = Left(S, Len(S) - 1) & " "
            End Select
            Declaration = Declaration & S
            LineNum = LineNum + 1
            S = CodeMod.Lines(LineNum, 1)
        Loop
        Declaration = SingleSpace(Declaration & S)
        GetProcedureDeclaration = Declaration
        
    
    End Function
    
    Private Function SingleSpace(ByVal Text As String) As String
        Dim Pos As String
        Pos = InStr(1, Text, Space(2), vbBinaryCompare)
        Do Until Pos = 0
            Text = Replace(Text, Space(2), Space(1))
            Pos = InStr(1, Text, Space(2), vbBinaryCompare)
        Loop
        SingleSpace = Text
    End Function

'You can call the ProcedureInfo function using code like the following:
    Sub ShowProcedureInfo()
        Dim VBAEditor As VBIDE.VBE
        Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim CodeMod As VBIDE.CodeModule
        Dim CompName As String
        Dim ProcName As String
        Dim ProcKind As VBIDE.vbext_ProcKind
        Dim PInfo As ProcInfo
        
        CompName = "modVBECode"
        ProcName = "ProcedureInfo"
        ProcKind = vbext_pk_Proc
        
        Set VBAEditor = Application.VBE
        Set VBProj = VBAEditor.ActiveVBProject
        Set VBComp = VBProj.VBComponents(CompName)
        Set CodeMod = VBComp.CodeModule
        
        PInfo = ProcedureInfo(ProcName, ProcKind, CodeMod)
        
        Debug.Print "ProcName: " & PInfo.ProcName
        Debug.Print "ProcKind: " & CStr(PInfo.ProcKind)
        Debug.Print "ProcStartLine: " & CStr(PInfo.ProcStartLine)
        Debug.Print "ProcBodyLine: " & CStr(PInfo.ProcBodyLine)
        Debug.Print "ProcCountLines: " & CStr(PInfo.ProcCountLines)
        Debug.Print "ProcScope: " & CStr(PInfo.ProcScope)
        Debug.Print "ProcDeclaration: " & PInfo.ProcDeclaration
    End Sub
    
'######################
'Searching For Text In A Module
'The CodeModule object has a Find method that you can use to search for text within the code module.
'The Find method accepts ByRef Long parameters.
'Upon input, these parameters specify the range of lines and column to search.
'On output, these values will point to the found text.
'To find the second and subsequent occurence of the text, you need to set the parameters to refer to the text
'following the found line and column.
'The Find method returns True or False indicating whether the text was found.

'The code below will search all of the code in Module1 and print a message listing each found occurrence.
'Note the values set with the SL, SC, EL, and EC variables. The code loops until the Found variable is False.

Sub SearchCodeModule()

On Error GoTo Err_Handler
    Dim VBAEditor As VBIDE.VBE
    Dim VBProj As VBIDE.VBProject
    Dim VBComp As VBIDE.VBComponent
    Dim CodeMod As VBIDE.CodeModule
    Dim FindWhat As String
    Dim Sl As Long ' start line
    Dim El As Long ' end line
    Dim Sc As Long ' start column
    Dim EC As Long ' end column
    Dim Found As Boolean
    Dim ProcName As String
    Dim ProcKind As VBIDE.vbext_ProcKind
    
    Dim strSearch As String, strResult As String
    
    strModule = "LibraryCode1"
    strSearch = "ggg"
    strResult = ""
    strProc = ""
    
    Set VBAEditor = Application.VBE
    Set VBProj = VBAEditor.ActiveVBProject
    Set VBComp = VBProj.VBComponents(strModule)
    Set CodeMod = VBComp.CodeModule
            
    With CodeMod
        Sl = 1
        El = .CountOfLines
        Sc = 1
        EC = 255
        Found = .Find(Target:=strSearch, StartLine:=Sl, StartColumn:=Sc, _
            EndLine:=El, EndColumn:=EC, _
            wholeword:=True, MatchCase:=False, PatternSearch:=False)
        Do Until Found = False
            strProc = .ProcOfLine(Sl, ProcKind)
            strResult = strResult & " Procedure: " & strProc & " - Line: " & CStr(Sl) & " Column: " & CStr(Sc) & vbNewLine
            
            El = .CountOfLines
            Sc = EC + 1
            EC = 255
            Found = .Find(Target:=strSearch, StartLine:=Sl, StartColumn:=Sc, _
                EndLine:=El, EndColumn:=EC, _
                wholeword:=True, MatchCase:=False, PatternSearch:=False)
                
            strProc = .ProcOfLine(Sl, ProcKind)
            strResult = strResult & " Procedure: " & strProc & vbTab & " - Line: " & CStr(Sl) & " Column: " & CStr(Sc) & vbNewLine
        Loop
    End With
    
    If strResult <> "" Then
        FormattedMsgBox "Search string '" & strSearch & "' found in module " & strModule & " at :" & _
            vbTab & "@" & strResult & "         @", vbInformation, "Search result"
    Else
        MsgBox "Search string '" & strSearch & "' not found in module " & strModule & ".   ", vbInformation, "Search result"
    End If
                
Exit_Handler:
    Exit Sub
        
Err_Handler:
    MsgBox "Error " & Err.Number & " in SearchCodeModule procedure : " & Err.Description
    GoTo Exit_Handler
End Sub
    
Public Sub SearchStringInModuleProcedure(strModule As String, strSearch As String)

On Error GoTo Err_Handler
    Dim VBAEditor As VBIDE.VBE
    Dim VBProj As VBIDE.VBProject
    Dim VBComp As VBIDE.VBComponent
    Dim CodeMod As VBIDE.CodeModule
   ' Dim FindWhat As String
    
    Dim Sl As Long ' start line
    Dim El As Long ' end line
    Dim Sc As Long ' start column
    Dim EC As Long ' end column
    Dim Found As Boolean
    Dim ProcName As String
    Dim ProcKind As VBIDE.vbext_ProcKind
        
    Dim strResult As String
    
    'strSearch = ""
    strResult = ""
    strProc = ""
    
    Set VBAEditor = Application.VBE
    Set VBProj = VBAEditor.ActiveVBProject
    Set VBComp = VBProj.VBComponents(strModule)
    Set CodeMod = VBComp.CodeModule
    
    With CodeMod
        Sl = 1
        El = .CountOfLines
        Sc = 1
        EC = 255
        Found = .Find(Target:=strSearch, StartLine:=Sl, StartColumn:=Sc, _
            EndLine:=El, EndColumn:=EC, _
            wholeword:=True, MatchCase:=False, PatternSearch:=False)
        Do Until Found = False
            strProc = .ProcOfLine(Sl, ProcKind)
            strResult = strResult & " Procedure: " & strProc & " - Line: " & CStr(Sl) & " Column: " & CStr(Sc) & vbNewLine
            'Debug.Print "Found at: Line: " & CStr(Sl) & " Column: " & CStr(Sc)
            El = .CountOfLines
            Sc = EC + 1
            EC = 255
            Found = .Find(Target:=strSearch, StartLine:=Sl, StartColumn:=Sc, _
                EndLine:=El, EndColumn:=EC, _
                wholeword:=True, MatchCase:=False, PatternSearch:=False)
                
            strProc = .ProcOfLine(Sl, ProcKind)
            strResult = strResult & " Procedure: " & strProc & " - Line: " & CStr(Sl) & " Column: " & CStr(Sc) & vbNewLine

        Loop
    End With
    
    If strResult <> "" Then
        FormattedMsgBox "Search string '" & strSearch & "' found in module " & strModule & " at :                       " & _
            vbTab & "@" & strResult & "                          @", vbInformation, "Search result"
    Else
        MsgBox "Search string '" & strSearch & "' not found in module " & strModule & ".   ", vbInformation, "Search result"
    End If
    
Exit_Handler:
    Exit Sub
        
Err_Handler:
    MsgBox "Error " & Err.Number & " in SearchStringInModuleProcedure procedure : " & Err.Description
    GoTo Exit_Handler
    
End Sub
        
'######################
'Testing If A VBComponent Exists

    Public Function VBComponentExists(VBCompName As String, Optional VBProj As VBIDE.VBProject = Nothing) As Boolean
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' This returns True or False indicating whether a VBComponent named
    ' VBCompName exists in the VBProject referenced by VBProj. If VBProj
    ' is omitted, the current VBProject is used.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Dim VBP As VBIDE.VBProject
        If VBProj Is Nothing Then
            Set VBP = VBProj
        End If
        On Error Resume Next
        VBComponentExists = CBool(Len(VBP.VBComponents(VBCompName).Name))
    
    End Function
    
'######################
'Total Code Lines In A Component Code Module

    Public Function TotalCodeLinesInVBComponent(VBComp As VBIDE.VBComponent) As Long
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' This returns the total number of code lines (excluding blank lines and
    ' comment lines) in the VBComponent referenced by VBComp.
    ' Returns -1 if the VBProject is locked.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        Dim N As Long
        Dim S As String
        Dim LineCount As Long
        
        If VBComp.Collection.Parent.Protection = vbext_pp_locked Then
            TotalCodeLinesInVBComponent = -1
            Exit Function
        End If
        
        With VBComp.CodeModule
            For N = 1 To .CountOfLines
                S = .Lines(N, 1)
                If Trim(S) = vbNullString Then
                    ' blank line, skip it
                ElseIf Left(Trim(S), 1) = "'" Then
                    ' comment line, skip it
                Else
                    LineCount = LineCount + 1
                End If
            Next N
        End With
        
        TotalCodeLinesInVBComponent = LineCount
        Debug.Print TotalCodeLinesInVBComponent
    End Function
    
'######################
'Total Lines In A Project

    Public Function TotalLinesInProject(Optional VBProj As VBIDE.VBProject = Nothing) As Long
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' This returns the total number of lines in all components of the VBProject
    ' referenced by VBProj. If VBProj is missing, the current VBProject is used
    ' Returns -1 if the VBProject is locked.
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
        
        Dim VBAEditor As VBIDE.VBE
        'Dim VBProj As VBIDE.VBProject
        Dim VBComp As VBIDE.VBComponent
        Dim LineCount As Long
        
        Set VBAEditor = Application.VBE
        
        If VBProj Is Nothing Then
            Set VBProj = VBAEditor.ActiveVBProject
        End If
                
     '   Debug.Print VBP.Protection
        
        If VBProj.Protection = vbext_pp_locked Then
            TotalLinesInProject = -1
            Exit Function
        End If
        
        For Each VBComp In VBProj.VBComponents
            LineCount = LineCount + VBComp.CodeModule.CountOfLines
        Next VBComp
        
        TotalLinesInProject = LineCount
        
        'Debug.Print TotalLinesInProject
    End Function
     
'#####################
'Not from Chip Pearson
Public Sub FindStringInModules(ByVal pSearchWord As String)

    Dim objComponent As VBComponent
    ' VBComponent requires reference to Microsoft Visual Basic for Applications Extensibility
    ' otherwise use late binding instead:
    'Dim objComponent As Object
    Dim strMessage As String
    Dim strModuleList As String

    strModuleList = vbNullString
    
    For Each objComponent In Application.VBE.ActiveVBProject.VBComponents
        'mdl.Find(strSearchText, lngSLine, lngSCol, lngELine, lngECol)
        If objComponent.CodeModule.Find(pSearchWord, 1, 1, -1, -1) = True Then
            strModuleList = strModuleList & "; " & objComponent.Name
        End If
    Next objComponent
    strMessage = "Text '" & pSearchWord & "' found in "
    If Len(strModuleList) > 0 Then
        strMessage = strMessage & "modules: " & Mid(strModuleList, 3)
    Else
        strMessage = strMessage & "no modules"
    End If
    'Debug.Print strMessage
End Sub
    
'#############################
'Not from Chip Pearson
'The following example will extract all of the code in the active project and display it in the immediate window.
'The declarations and procedures have been provided their own iterators to enable working with a module in sections.

'It is also using the Lines collection to display the single line with which the index represents.

Public Sub ListCode()

    Dim Component As Object
    Dim Index As Long

    For Each Component In Application.VBE.ActiveVBProject.VBComponents

        With Component.CodeModule

            'The Declarations
            For Index = 1 To .CountOfDeclarationLines
              '  Debug.Print .Lines(Index, 1)
            Next Index

            'The Procedures
            For Index = .CountOfDeclarationLines + 1 To .CountOfLines
                Debug.Print .Lines(Index, 1)
            Next Index

        End With

    Next Component

End Sub

Function GetCodeLinesInComponent()

'CR 05/03/2019

     Dim VBAEditor As VBIDE.VBE
     Dim VBProj As VBIDE.VBProject
     Dim VBComp As VBIDE.VBComponent
 '   Dim CodeModule As VBIDE.CodeModule
        
    Set VBAEditor = Application.VBE
    Set VBProj = VBAEditor.ActiveVBProject
    Set VBComp = VBProj.VBComponents("Form_frmMain") 'modify as appropriate
        
    GetCodeLinesInComponent = VBComp.CodeModule.CountOfLines

    Debug.Print GetCodeLinesInComponent

    
End Function



    
 


  
 
 


